%{
/* DynGenPar: Dynamic Generalized Parser - Naproche formula lexer
   Copyright (C) 2010 Kevin Kofler <kevin.kofler@chello.at>

   Support by the Austrian Science Fund FWF under contract number P20631
   is gratefully acknowledged.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
%}

 /* definitions: */
%option batch
%option never-interactive
%option noinput
%option nounput
%option noyywrap
%option c++
%option prefix="NaprocheFormula"
%option nodefault
%option warn

%{
#include "naproche-cats.h"
%}

%%
 /* rules: */
"$" {return DOLLAR;}
"(" {return LPAREN;}
")" {return RPAREN;}
m {return VAR_M;}
n {return VAR_N;}
t {return VAR_T;}
u {return VAR_U;}
v {return VAR_V;}
w {return VAR_W;}
x {return VAR_X;}
y {return VAR_Y;}
z {return VAR_Z;}
"\\emptyset" {return CST_EMPTYSET;}
c {return CST_C;}
0 {return CST_0;}
1 {return CST_1;}
2 {return CST_2;}
3 {return CST_3;}
4 {return CST_4;}
5 {return CST_5;}
f {return FUN_F;}
f2 {return FUN_F2;}
g {return FUN_G;}
h {return FUN_H;}
"*" {return FUN_STAR;}
"+" {return FUN_PLUS;}
"^" {return FUN_CARET;}
"'" {return FUN_PRIME;}
[Ss]ucc {return FUN_SUCC;}
"\\cup" {return FUN_CUP;}
"\\cap" {return FUN_CAP;}
"=" {return REL_EQ;}
"\\neq" {return REL_NEQ;}
"<" {return REL_LT;}
">" {return REL_GT;}
"\\leq" {return REL_LEQ;}
"\\geq" {return REL_GEQ;}
"\\in" {return REL_IN;}
R {return REL_R;}
r {return REL_SMALL_R;}
M {return REL_M;}
[Oo]rd {return REL_ORD;}
[Tt]rans {return REL_TRANS;}
[Cc]ontradiction {return REL_CONTRADICTION;}
"\\subset" {return REL_SUBSET;}
"\\supset" {return REL_SUPSET;}
"\\forall" {return QUANT_FORALL;}
"\\exists" {return QUANT_EXISTS;}
"\\neg" {return LOG_NEG;}
"\\wedge" {return LOG_AND;}
"\\and" {return LOG_AND;}
"\\vee" {return LOG_OR;}
"\\or" {return LOG_OR;}
"\\rightarrow" {return LOG_IMPLIES;}
"\\implies" {return LOG_IMPLIES;}
"\\leftrightarrow" {return LOG_EQUIVALENT;}
"\\equivalent" {return LOG_EQUIVALENT;}
[ \t\n.:,] ; /* skip whitespace and some punctuation */
. {return SCAN_ERROR;} /* invalid character */
%%
/* no subroutines */
