%{
/* DynGenPar: Dynamic Generalized Parser - Naproche lexer
   Copyright (C) 2009-2010 Kevin Kofler <kevin.kofler@chello.at>

   Support by the Austrian Science Fund FWF under contract number P20631
   is gratefully acknowledged.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
%}

 /* definitions: */
%option batch
%option never-interactive
%option noinput
%option nounput
%option noyywrap
%option c++
%option prefix="Naproche"
%option nodefault
%option warn

%{
#include "naproche-cats.h"
%}

%%
 /* rules: */
\$[^$]*\$  {return FORMULA;}
[tT]"heorem." {return THEOREM;}
[pP]"roof." {return PROOF;}
[qQ]"ed." {return QED;}
[lL]"emma." {return LEMMA;}
[dD]efine {return DEFINE;}
iff {return IFF;}
if,?" and only if" {return IFF;}
[aA]"ssume that" {return ASSUME_THAT;}
[aA]"ssume for a contradiction that" {return ASSUME_CONTR;}
[lL]et {return LET;}
[cC]onsider {return CONSIDER;}
[tT]hus {return THUS;}
[cC]"ontradiction." {return CONTRADICTION;}
"for all" {return FOR_ALL;}
"," {return COMMA;}
[tT]"here is an" {return THERE_IS_AN;}
"such that" {return SUCH_THAT;}
implies {return IMPLIES;}
[nN]ot {return NOT;}
[tT]hen {return THEN;}
[hH]ence {return HENCE;}
[rR]"ecall that" {return RECALL_THAT;}
[bB]ut {return BUT;}
[iI]"n particular" {return IN_PARTICULAR;}
[oO]"bserve that" {return OBSERVE_THAT;}
[tT]"ogether we have" {return TOGETHER_WE_HAVE;}
[sS]o {return SO;}
[ \t\n.:] ; /* skip whitespace and some punctuation */
. {return SCAN_ERROR;} /* invalid character */
%%
/* no subroutines */
